"""Module for custom warnings."""


class NoFontFamily(Warning):
    """Warning raised when no font family is found."""

    pass


class NoRegularStyle(Warning):
    """Warning raised when no regular style is found."""

    pass
